@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl 4821 2009-05-12 10:23:34Z sysjkf $
+#   $Id: ConfigDefs.pl 4972 2009-12-04 14:39:05Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -218,6 +218,7 @@ mcpmodifysubject		= mcpmodifysubject
 sizemodifysubject		= sizemodifysubject
 sizesubjecttext			= sizesubjecttext
 spamassassintempdir		= spamassassintemporarydir
+spaminfected			= VirusNamesWhichAreSpam
 spammodifysubject		= spammodifysubject
 spamscorenotstars		= spamscorenumberinsteadofstars
 spamstars			= spamscore
@@ -393,6 +394,7 @@ SpamAssassinLocalRulesDir
 SpamAssassinLocalStateDir	
 SpamAssassinDefaultRulesDir	
 SpamAssassinInstallPrefix	
+SpamInfected		Sane*UNOFFICIAL
 SpamStarsCharacter	s
 MCPMaxSpamAssassinSize		100000
 MCPSpamAssassinUserStateDir
@@ -447,6 +449,7 @@ LookForUU		0	no	0	yes	1
 MultipleHeaders		append	append	append	replace	replace	add	add
 NameModifySubject	start	no	0	yes	1	start	start	end	end
 NoticeFullHeaders	1	no	0	yes	1
+RejectMessage		0	no	0	yes	1
 ScannedModifySubject	0	no	0	yes	1	start	start	end	end
 SendNotices		1	no	0	yes	1
 SignAlreadyScanned	0	no	0	yes	1
@@ -572,6 +575,7 @@ ScoreFormat			%d
 Sendmail2			/usr/sbin/sendmail
 SpamHeader			X-MailScanner-SpamCheck:
 SpamList
+SpamVirusHeader			X-MailScanner-SpamVirus-Report:
 MCPSubjectText			{MCP?}
 SpamSubjectText			{Spam?}
 SpamStarsHeader			X-MailScanner-SpamScore:
@@ -621,7 +625,6 @@ QuarantineInfections	1	no	0	yes	1
 QuarantineModifiedBody	0	no	0	yes	1
 QuarantineSilent	0	no	0	yes	1
 QuarantineWholeMessage	0	no	0	yes	1
-RejectMessage		0	no	0	yes	1
 ReplaceTNEF		2	no	0	add	1	replace	2
 sadecodebins		0	no	0	yes	1
 ScanMail		1	no	0	yes	1
