@@ -765,7 +765,7 @@ my($hostname);
 
 my $ConfFile = $ARGV[0];
 $ConfFile = $ARGV[1] if $ConfFile =~ /^-+/i;
-$ConfFile = '/opt/MailScanner/etc/MailScanner.conf' unless $ConfFile && -f $ConfFile;
+$ConfFile = '/etc/MailScanner/MailScanner.conf' unless $ConfFile && -f $ConfFile;
 
 my ($AccessDB, $Refusal, $my_mta); # Generalized here for Multiple MTA support
 $my_mta = lc(MailScanner::Config::QuickPeek($ConfFile, 'mta')) if $ConfFile &&
