@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm 4842 2009-05-28 10:39:00Z sysjkf $
+#   $Id: MessageBatch.pm 4972 2009-12-04 14:39:05Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ use DBI qw(:sql_types);
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4842 $, 10;
+$VERSION = substr q$Revision: 4972 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -333,6 +333,7 @@ sub HandleHam {
 
 # Reject messages that come from people we want to reject. Send nice report
 # instead.
+# 2009-12-04 Changed from an "All" to a "First" match rule. Much more useful.
 sub RejectMessages {
   my $this = shift;
 
@@ -343,7 +344,7 @@ sub RejectMessages {
     next if $message->{deleted};
     #print STDERR "May reject message $id\n";
     $message->RejectMessage()
-      if MailScanner::Config::Value('rejectmessage',$message) !~ /0/;
+      if MailScanner::Config::Value('rejectmessage',$message);
   }
 }
 
@@ -432,6 +433,19 @@ sub HandleNonMCP {
 }
 
 
+# Return true if all the messages in the batch are deleted!
+# Return false otherwise.
+sub Empty {
+  my $this = shift;
+
+  my($id, $message);
+  while(($id,$message) = each %{$this->{messages}}) {
+    return 0 unless $message->{deleted};
+  }
+  return 1;
+}
+
+
 # Deliver the messages that aren't to be scanned.
 # Uses the "virusscanme" property to determine this.
 # This does not add the clean sig or anything like that.
@@ -451,6 +465,22 @@ sub DeliverUnscanned {
       push @messages, $message;
       next;
     }
+  }
+
+  # Note this passes a list now, not a ref to a list
+  MailScanner::Mail::TellAbout(@messages);
+
+  MailScanner::Log::InfoLog("Unscanned: Delivered %d messages",
+                            scalar(@messages)) if @messages;
+}
+
+sub DeliverUnscanned2 {
+  my $this = shift;
+  
+  my($OutQ, @messages, $id, $message);
+  
+  while(($id,$message) = each %{$this->{messages}}) {
+    next if $message->{deleted} || $message->{dontdeliver};
 
     if (!$message->{scanme}) {
       #print STDERR "Delivering unscanned message $id\n";
@@ -516,6 +546,9 @@ sub Explode {
 sub VirusScan {
   my $this = shift;
 
+  # Don't do any virus scanning at all if the batch is empty!
+  return if $this->Empty();
+
   #MailScanner::Log::InfoLog("Other Checks: Starting");
   my $others  = MailScanner::SweepOther::ScanBatch($this, 'scan');
   MailScanner::Log::NoticeLog("Other Checks: Found %d problems", $others+0)
@@ -818,6 +851,7 @@ sub FindSilentAndNoisyInfections {
   while(($id, $message) = each %{$this->{messages}}) {
     next if !$message->{infected};
     next if $message->{deleted} && !$message->{stillwarn};
+    next if $message->{spamvirusreport}; # Silent exclude Spam-Viruses
     $message->FindSilentAndNoisyInfections();
   }
 }
@@ -825,7 +859,7 @@ sub FindSilentAndNoisyInfections {
 
 # Deliver all the "silent" and not noisy infected messages,
 # and mark them for deletion from the queue.
-sub DeliverOrDeleteSilent {
+sub DeliverOrDeleteSilentExceptSpamViruses {
   my $this = shift;
 
   my($id, $message, @messages);
@@ -1321,5 +1355,29 @@ sub DecrementProcDB {
   #$updatesth->finish;
 }
 
+# Delete all the messages from the batch. Used when none of the virus
+# scanners worked and we have been told to not process mail when the
+# scanners are all dead.
+sub DropBatch {
+  my($this) = @_;
+
+  return unless $this;
+
+  while(my($id, $message) = each %{$this->{messages}}) {
+    $message->{deleted} = 1;
+    $message->{gonefromdisk} = 1; # Don't try to delete the original
+    $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+  }
+
+  # This is a very good place for a snooze.
+  # If the entire batch has been abandoned, it will instantly loop all the
+  # way around and try to pick up the same messages into a new batch.
+  # This will cause the CPU load and disk load to go through the roof as it
+  # will run away with itself trying to collect batches and them drop them
+  # again.
+  sleep 10;
+}
+
+
 1;
 
