@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Sendmail.pm 4443 2008-05-08 00:22:17Z sysjkf $
+#   $Id: Sendmail.pm 4813 2009-05-08 10:48:31Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -37,11 +37,12 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 
 use DirHandle;
 use Encode;
+use DBI;
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4443 $, 10;
+$VERSION = substr q$Revision: 4813 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -271,7 +272,7 @@ my($sed) = "/bin/sed";
 
   # Change for V4: returns lower-case $from and @to
   sub ReadQf {
-    my($this, $message) = @_;
+    my($this, $message, $getipfromheader) = @_;
 
     my($RQf) = $message->{store}{inhhandle};
 
@@ -279,6 +280,7 @@ my($sed) = "/bin/sed";
     my($ip, $Rline);
     my($Line, $Flags);
     my($RFound, $SFound, $IPFound);
+    my(@rcvdiplist);
 
     #$message->{store}->print();
 
@@ -356,10 +358,37 @@ my($sed) = "/bin/sed";
         $message->{subject} = $1;
         $InSubject = 1;
       }
+      # Non-greedy match to pull out 1st IP address on the line
+      if ($Line =~ /^Received:/i) {
+        my $rcvdip = '127.0.0.1';
+        if ($Line =~ /^Received: .+?\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+          $rcvdip = $1;
+          #unless ($read1strcvd) {
+          #  $ipfromheader = $1;
+          #  $read1strcvd = 1;
+          #}
+        # Non-greedy match to pull out 1st IP address on the line
+        } elsif ($Line =~ /^Received: .+?\[([\dabcdef.:]+)\]/i) {
+          $rcvdip = $1;
+          #unless ($read1strcvd) {
+          #  $ipfromheader = $1;
+          #  $read1strcvd = 1;
+          #}
+        }
+        push @rcvdiplist, $rcvdip;
+      }
     }
     # Not every qf file defined an IP address if it is a bounce.
     # So provide an IP if we haven't found one.
     $message->{clientip} = '0.0.0.0' unless $IPFound;
+    # If we were told to read the IP from the header, and it was there.
+    # Use the last element if there weren't enough IP addresses.
+    $getipfromheader = @rcvdiplist if $getipfromheader>@rcvdiplist;
+    # If they wanted the 2nd Received from address, give'em element 1 of list
+    $message->{clientip} = $rcvdiplist[$getipfromheader-1] if
+      $getipfromheader>0;
+    #$message->{clientip} = $ipfromheader
+    #  if $read1strcvd && $getipfromheader && $ipfromheader ne "";
 
     # Decode ISO subject lines into UTF8
     # Needed for UTF8 support in MailWatch 2.0
@@ -435,15 +464,29 @@ my($sed) = "/bin/sed";
 
   sub AddHeader {
     my($this, $message, $newkey, $newvalue) = @_;
-    push @{$message->{metadata}}, "H$newkey $newvalue";
+    # DKIM: Add header before first H line if adding at top
+    if ($message->{newheadersattop}) {
+      unshift @{$message->{metadata}}, "H$newkey $newvalue";
+    } else {
+      push @{$message->{metadata}}, "H$newkey $newvalue";
+    }
   }
 
   sub DeleteHeader {
     my($this, $message, $key) = @_;
 
-    my($linenum);
+    my $usingregexp = ($key =~ s/^\/(.*)\/$/$1/)?1:0;
+
+    # Add a colon if they forgot it.
+    $key .= ':' unless $usingregexp || $key =~ /:$/;
+    # If it's not a regexp, then anchor it and sanitise it.
+    $key = '^' . quotemeta($key) unless $usingregexp;
+
+    my($linenum, $line);
     for ($linenum=0; $linenum<@{$message->{metadata}}; $linenum++) {
-      next unless $message->{metadata}[$linenum] =~ /^H(\?[^?]*\?)?$key/i;
+      $line = $message->{metadata}[$linenum];
+      next unless $line =~ s/^H(\?[^?]*\?)?//;
+      next unless $line =~ /$key/i;
       # Have found the right line
       splice(@{$message->{metadata}}, $linenum, 1);
       # Now delete the continuation lines
@@ -478,7 +521,8 @@ my($sed) = "/bin/sed";
   sub ReplaceHeader {
     my($this, $message, $key, $newvalue) = @_;
 
-    $this->DeleteHeader($message, $key);
+    # DKIM: Don't do DeleteHeader if only adding headers at top
+    $this->DeleteHeader($message, $key) unless $message->{dkimfriendly};
     $this->AddHeader($message, $key, $newvalue);
   }
 
@@ -842,6 +886,7 @@ my($sed) = "/bin/sed";
     my(%ModDate, $mta, $file, $tmpdate, $invalidfiles);
     my(@SortedFiles, $id, $newmessage, @queuedirnames);
     my($batchempty, $CriticalQueueSize, $headerfileumask);
+    my($getipfromheader);
 
     # Old code left over from single queue dir
     #$queuedirname = $global::MS->{inq}{dir};
@@ -860,6 +905,7 @@ my($sed) = "/bin/sed";
     # If there are too many messages in the queue, start processing in
     # directory storage order instead of date order.
     $CriticalQueueSize = MailScanner::Config::Value('criticalqueuesize');
+    $getipfromheader = MailScanner::Config::Value('getipfromheader');
 
     # Set what we will need the umask to be
     $headerfileumask = $global::MS->{work}->{fileumask};
@@ -977,6 +1023,7 @@ my($sed) = "/bin/sed";
       }
 
       $batchempty = 1;
+      my $maxattempts = MailScanner::Config::Value('procdbattempts');
 
       # Keep going until end of dir or have reached every imposed limit. This
       # now processes the files oldest first to make for fairer queue cleanups.
@@ -1009,13 +1056,72 @@ my($sed) = "/bin/sed";
         #print STDERR "Adding $id to batch from $queuedirname\n";
 
         # Lock and read the qf file. Skip this message if the lock fails.
-        $newmessage = MailScanner::Message->new($id, $queuedirname);
-        if ($newmessage eq 'INVALID') {
+        $newmessage = MailScanner::Message->new($id, $queuedirname,
+                                                $getipfromheader);
+        if ($newmessage && $newmessage->{INVALID}) {
+        #if ($newmessage eq 'INVALID') {
           $invalidfiles .= "$id ";
+          undef $newmessage;
           next;
         }
         next unless $newmessage;
 
+        # JKF 20090301 Skip this message if $id has been scanned
+        # too many times.
+        # JKF 20090301 Read the number of times this message id has
+        # been processed. If over the limit, then ignore it.
+        #print STDERR "maxattempts = $maxattempts\n";
+        if ($maxattempts) {
+          my $nexttime = time + 120 + int(rand(240)); # 4 +- 2 minutes
+          #my $nexttime = time + 10 + int(rand(20)); # 4 +- 2 minutes
+          my @attempts = $MailScanner::ProcDBH->selectrow_array(
+                           $MailScanner::SthSelectRows, undef, $id);
+          #my @attempts = $MailScanner::ProcDBH->selectrow_array(
+          #"SELECT id,count,nexttime FROM processing WHERE (id='" . $id . "')");
+          #print STDERR "id       = \"$attempts[0]\"\n";
+          #print STDERR "count    = \"$attempts[1]\"\n";
+          #print STDERR "nexttime = \"$attempts[2]\"\n";
+          if (@attempts && $attempts[1]>=$maxattempts) {
+            MailScanner::Log::WarnLog("Warning: skipping message %s as it has been attempted too many times", $id);
+            # JKF 20090301 next;
+            # Instead of just skipping it, quarantine it and notify
+            # the local postmaster.
+            $newmessage->QuarantineDOS();
+            #print STDERR "Moving $attempts[0], $attempts[1], $attempts[2] into archive table\n";
+            $MailScanner::SthDeleteId->execute($id);
+            #$MailScanner::ProcDBH->do(
+            #  "DELETE FROM processing WHERE id='" . $id . "'");
+            $MailScanner::SthInsertArchive->execute($attempts[0], $attempts[1], $attempts[2]);
+            #$MailScanner::ProcDBH->do(
+            #  "INSERT INTO archive (id, count, nexttime) " .
+            #  "VALUES ('$attempts[0]', $attempts[1], $attempts[2])");
+          } elsif (defined $attempts[1]) {
+            # We have tried this message before
+            if (time>=$attempts[2]) {
+              # Time for next attempt has arrived
+              $MailScanner::SthIncrementId->execute($nexttime, $id);
+              #$MailScanner::ProcDBH->do(
+              #  "UPDATE processing SET count=count+1, nexttime=$nexttime " .
+              #  " WHERE id='" . $id . "'");
+              MailScanner::Log::InfoLog(
+                "Making attempt %d at processing message %s",
+                $attempts[1]+1, $id);
+              #print STDERR "Incremented $id\n";
+            } else {
+              # Not time for next attempt yet, so ignore the message
+              $newmessage->DropFromBatch();
+              next;
+            }
+          } else {
+            # We have never seen this message before
+            $MailScanner::SthInsertProc->execute($id, 1, $nexttime);
+            #$MailScanner::ProcDBH->do(
+            #  "INSERT INTO processing (id, count, nexttime) " .
+            #  "VALUES ('" . $id . "', 1, $nexttime)");
+            #print STDERR "Inserted $id\n";
+          }
+        }
+
         $batch->{messages}{"$id"} = $newmessage;
         $newmessage->{mtime} = $mtime;
         $batchempty = 0;
@@ -1042,6 +1148,7 @@ my($sed) = "/bin/sed";
           # single-file archiving of messages.
           $newmessage->WriteHeaderFile(); # Write the file of headers
         }
+        # JKF 20090301 Add $id to the list of messages being processed.
       }
       umask 0077; # Safety net as stopped creating files now
 
